@@ -12,6 +12,11 @@ from faster_whisper.audio import decode_audio
 from faster_whisper.feature_extractor import FeatureExtractor
 from faster_whisper.tokenizer import Tokenizer
 from faster_whisper.utils import download_model
+from faster_whisper.vad import (
+    SpeechTimestampsMap,
+    collect_chunks,
+    get_speech_timestamps,
+)
 
 
 class Word(NamedTuple):
@@ -26,6 +31,8 @@ class Segment(NamedTuple):
     end: float
     text: str
     words: Optional[List[Word]]
+    avg_log_prob: float
+    no_speech_prob: float
 
 
 class AudioInfo(NamedTuple):
@@ -68,9 +75,10 @@ class WhisperModel:
         """Initializes the Whisper model.
 
         Args:
-          model_size_or_path: Size of the model to use (e.g. "large-v2", "small", "tiny.en", etc.)
-            or a path to a converted model directory. When a size is configured, the converted
-            model is downloaded from the Hugging Face Hub.
+          model_size_or_path: Size of the model to use (tiny, tiny.en, base, base.en,
+            small, small.en, medium, medium.en, large-v1, or large-v2) or a path to a converted
+            model directory. When a size is configured, the converted model is downloaded
+            from the Hugging Face Hub.
           device: Device to use for computation ("cpu", "cuda", "auto").
           device_index: Device ID to use.
             The model can also be loaded on multiple GPUs by passing a list of IDs
@@ -149,6 +157,8 @@ class WhisperModel:
         word_timestamps: bool = False,
         prepend_punctuations: str = "\"'“¿([{-",
         append_punctuations: str = "\"'.。,，!！?？:：”)]}、",
+        vad_filter: bool = False,
+        vad_parameters: Optional[dict] = None,
     ) -> Tuple[Iterable[Segment], AudioInfo]:
         """Transcribes an input file.
 
@@ -164,13 +174,13 @@ class WhisperModel:
           length_penalty: Exponential length penalty constant.
           temperature: Temperature for sampling. It can be a tuple of temperatures,
             which will be successively used upon failures according to either
-            `compression_ratio_threshold` or `logprob_threshold`.
+            `compression_ratio_threshold` or `log_prob_threshold`.
           compression_ratio_threshold: If the gzip compression ratio is above this value,
             treat as failed.
           log_prob_threshold: If the average log probability over sampled tokens is
             below this value, treat as failed.
           no_speech_threshold: If the no_speech probability is higher than this value AND
-            the average log probability over sampled tokens is below `logprob_threshold`,
+            the average log probability over sampled tokens is below `log_prob_threshold`,
             consider the segment as silent.
           condition_on_previous_text: If True, the previous output of the model is provided
             as a prompt for the next window; disabling may make the text inconsistent across
@@ -189,6 +199,11 @@ class WhisperModel:
             with the next word
           append_punctuations: If word_timestamps is True, merge these punctuation symbols
             with the previous word
+          vad_filter: Enable the voice activity detection (VAD) to filter out parts of the audio
+            without speech. This step is using the Silero VAD model
+            https://github.com/snakers4/silero-vad.
+          vad_parameters: Dictionary of Silero VAD parameters (see available parameters and
+            default values in the function `get_speech_timestamps`).
 
         Returns:
           A tuple with:
@@ -202,9 +217,17 @@ class WhisperModel:
             )
 
         duration = audio.shape[0] / self.feature_extractor.sampling_rate
+
+        if vad_filter:
+            vad_parameters = {} if vad_parameters is None else vad_parameters
+            speech_chunks = get_speech_timestamps(audio, **vad_parameters)
+            audio = collect_chunks(audio, speech_chunks)
+        else:
+            speech_chunks = None
+
         features = self.feature_extractor(audio)
 
-        whisper_encoder = WhisperEncoder(self.model)
+        encoder_output = None
 
         if language is None:
             if not self.model.is_multilingual:
@@ -212,7 +235,7 @@ class WhisperModel:
                 language_probability = 1
             else:
                 segment = features[:, : self.feature_extractor.nb_max_frames]
-                encoder_output = whisper_encoder(0, segment)
+                encoder_output = self.encode(segment)
                 results = self.model.detect_language(encoder_output)
                 language_token, language_probability = results[0][0]
                 language = language_token[2:-2]
@@ -241,7 +264,7 @@ class WhisperModel:
             initial_prompt=initial_prompt,
             prefix=prefix,
             suppress_blank=suppress_blank,
-            suppress_tokens=suppress_tokens,
+            suppress_tokens=get_suppressed_tokens(tokenizer, suppress_tokens),
             without_timestamps=without_timestamps,
             max_initial_timestamp=max_initial_timestamp,
             word_timestamps=word_timestamps,
@@ -249,7 +272,12 @@ class WhisperModel:
             append_punctuations=append_punctuations,
         )
 
-        segments = self.generate_segments(features, whisper_encoder, tokenizer, options)
+        segments = self.generate_segments(features, tokenizer, options, encoder_output)
+
+        if speech_chunks:
+            segments = restore_speech_timestamps(
+                segments, speech_chunks, self.feature_extractor.sampling_rate
+            )
 
         audio_info = AudioInfo(
             language=language,
@@ -262,9 +290,9 @@ class WhisperModel:
     def generate_segments(
         self,
         features: np.ndarray,
-        whisper_encoder: "WhisperEncoder",
         tokenizer: Tokenizer,
         options: TranscriptionOptions,
+        encoder_output: Optional[ctranslate2.StorageView] = None,
     ) -> Iterable[Segment]:
         content_frames = features.shape[-1] - self.feature_extractor.nb_max_frames
         seek = 0
@@ -289,10 +317,11 @@ class WhisperModel:
                 tokenizer,
                 previous_tokens,
                 without_timestamps=options.without_timestamps,
-                prefix=options.prefix,
+                prefix=options.prefix if seek == 0 else None,
             )
 
-            encoder_output = whisper_encoder(seek, segment)
+            if encoder_output is None:
+                encoder_output = self.encode(segment)
 
             result, avg_log_prob, temperature = self.generate_with_fallback(
                 encoder_output, prompt, tokenizer, options
@@ -419,6 +448,8 @@ class WhisperModel:
                     if seek_shift > 0:
                         seek = previous_seek + seek_shift
 
+            encoder_output = None
+
             for segment in current_segments:
                 tokens = segment["tokens"]
                 text = tokenizer.decode(tokens)
@@ -437,8 +468,20 @@ class WhisperModel:
                         if options.word_timestamps
                         else None
                     ),
+                    avg_log_prob=avg_log_prob,
+                    no_speech_prob=result.no_speech_prob,
                 )
 
+    def encode(self, features: np.ndarray) -> ctranslate2.StorageView:
+        # When the model is running on multiple GPUs, the encoder output should be moved
+        # to the CPU since we don't know which GPU will handle the next job.
+        to_cpu = self.model.device == "cuda" and len(self.model.device_index) > 1
+
+        features = np.expand_dims(features, 0)
+        features = get_ctranslate2_storage(features)
+
+        return self.model.encode(features, to_cpu=to_cpu)
+
     def generate_with_fallback(
         self,
         encoder_output: ctranslate2.StorageView,
@@ -660,35 +703,34 @@ class WhisperModel:
         ]
 
 
-class WhisperEncoder:
-    """Helper class to cache and reuse the encoder output."""
+def restore_speech_timestamps(
+    segments: Iterable[Segment],
+    speech_chunks: List[dict],
+    sampling_rate: int,
+) -> Iterable[Segment]:
+    ts_map = SpeechTimestampsMap(speech_chunks, sampling_rate)
 
-    def __init__(self, model: ctranslate2.models.Whisper):
-        self.model = model
+    for segment in segments:
+        if segment.words:
+            words = []
+            for word in segment.words:
+                # Ensure the word start and end times are resolved to the same chunk.
+                chunk_index = ts_map.get_chunk_index(word.start)
+                word = word._replace(
+                    start=ts_map.get_original_time(word.start, chunk_index),
+                    end=ts_map.get_original_time(word.end, chunk_index),
+                )
+                words.append(word)
+        else:
+            words = segment.words
 
-        # When the model is running on multiple GPUs, the encoder output should be moved
-        # to the CPU since we don't know which GPU will handle the next job.
-        self.cache_on_cpu = len(model.device_index) > 1
+        segment = segment._replace(
+            start=ts_map.get_original_time(segment.start),
+            end=ts_map.get_original_time(segment.end),
+            words=words,
+        )
 
-        self.last_seek = -1
-        self.last_output = None
-
-    def __call__(self, seek: int, features: np.ndarray) -> ctranslate2.StorageView:
-        if self.last_seek == seek:
-            return self.last_output
-
-        features = np.expand_dims(features, 0)
-        features = get_ctranslate2_storage(features)
-
-        output = self.model.encode(features, to_cpu=self.cache_on_cpu)
-
-        if self.last_output is not None:
-            del self.last_output
-
-        self.last_seek = seek
-        self.last_output = output
-
-        return output
+        yield segment
 
 
 def get_ctranslate2_storage(segment: np.ndarray) -> ctranslate2.StorageView:
@@ -702,6 +744,27 @@ def get_compression_ratio(text: str) -> float:
     return len(text_bytes) / len(zlib.compress(text_bytes))
 
 
+def get_suppressed_tokens(tokenizer, suppress_tokens):
+    if not suppress_tokens or -1 in suppress_tokens:
+        return suppress_tokens
+
+    suppress_tokens = list(suppress_tokens)
+
+    # Ensure the following special tokens are suppressed when the user does
+    # not use the default set (-1).
+    suppress_tokens.extend(
+        [
+            tokenizer.transcribe,
+            tokenizer.translate,
+            tokenizer.sot,
+            tokenizer.sot_prev,
+            tokenizer.sot_lm,
+        ]
+    )
+
+    return sorted(set(suppress_tokens))
+
+
 def merge_punctuations(alignment: List[dict], prepended: str, appended: str):
     # merge prepended punctuations
     i = len(alignment) - 2
